from typing import Any, Dict

from kindtool import __version__, runner, templates, kindfile

class CmdUp:
    def __init__(self, tpl: templates.Templates) -> None:
        self._tpl = tpl
        self._kindfile = kindfile.Kindfile(tpl)
        self._runner = runner.Runner()

    def run(self) -> None:
        self._kindfile.throw_if_no_kindfile_found()

        cluster_name = self._kindfile.cluster_name()
        if self._runner.kind_is_running(cluster_name):
            print(f"cluster {cluster_name} is already running")
            return

        self._create_content()

        if self._kindfile.has_internal_registry():
            script = "internal-registry-create.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                print(f"error running: {script}")
                return

        args = [
            "create",
            "cluster",
            "--config", self._kindfile.config_yaml(),
            "--name", self._kindfile.cluster_name()
        ]

        if self._kindfile.has_local_kubeconfig():
            args.append("--kubeconfig")
            args.append(self._kindfile.kubeconfig())

        if not self._runner.kind(args):
            return "can't start the cluster"

        if self._kindfile.has_internal_registry():
            script = "internal-registry-connect.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                return f"error running: {script}"

        if self._kindfile.has_ingress():
            script = "ingress-install.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                return f"error running: {script}"

        if self._kindfile.has_metallb():
            script = "metallb-install.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                return f"error running: {script}"
            script = "update-metallb-ipaddresspool.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                return f"error running: {script}"


    def _create_content(self) -> str:
        self._tpl.delete_scripts_dir()
        self._tpl.delete_config_dir()

        cfg_data = self._kindfile.data()

        self._render_tpl_configs(cfg_data)

    def _render_tpl_configs(self, cfg_data: dict[str, str]) -> None:
        self._tpl.render_template(cfg_data, "j2/config.j2.yaml", ".kind/config")
        self._tpl.render_template(cfg_data, "j2/dot_gitignore.j2", ".kind", ".gitignore")

        key = "internal_registry"
        if key in cfg_data and cfg_data[key]:
            self._tpl.render_template(cfg_data, "j2/internal-registry-connect.j2.sh", ".kind/scripts", "", 0o0755)
            self._tpl.render_template(cfg_data, "j2/internal-registry-create.j2.sh", ".kind/scripts", "", 0o0755)
            self._tpl.render_template(cfg_data, "j2/internal-registry-delete.j2.sh", ".kind/scripts", "", 0o0755)
            self._tpl.render_template(cfg_data, "j2/internal-registry.j2.yaml", ".kind/config")

        key = "ingress"
        if key in cfg_data and cfg_data[key]:
            self._tpl.render_template(cfg_data, "j2/ingress-install.j2.sh", ".kind/scripts", "", 0o0755)

        key = "metallb"
        if key in cfg_data and cfg_data[key]:
            self._tpl.render_template(cfg_data, "j2/metallb-install.j2.sh", ".kind/scripts", "", 0o0755)
            self._tpl.render_template(cfg_data, "j2/update-metallb-ipaddresspool.j2.sh", ".kind/scripts", "", 0o0755)
            self._tpl.render_template(cfg_data, "metallb-config.tpl.yaml", ".kind/config")

        return None


from kindtool import templates
import os

class CmdInit:
    def __init__(self, tpl: templates.Templates) -> None:
        self._tpl = tpl

    def create_content(self) -> None:
        kind_filename = self._tpl.get_kindfile()
        if os.path.exists(kind_filename):
            raise FileExistsError(f"kindfile.yaml exists {kind_filename}")

        self._tpl.copy_file(tpl_filename="kindfile.yaml")



from kindtool import __version__, runner, templates, kindfile

import os

class CmdShell:
    def __init__(self, tpl: templates.Templates, pod: str, namespace: str, image: str, command: str) -> None:
        self._tpl = tpl
        inject = {}
        inject["shell_podname"] = pod
        inject["shell_namespace"] = namespace
        inject["shell_image"] = image
        inject["shell_command"] = command
        self._kindfile = kindfile.Kindfile(tpl, inject)
        self._runner = runner.Runner()

    def create_or_attach(self) -> None:
        self._kindfile.throw_if_no_kindfile_found()

        cluster_name = self._kindfile.cluster_name()
        if not self._runner.kind_is_running(cluster_name):
            print(f"cluster {cluster_name} is no running")
            return

        self._create_content()

        script = "shell-install.sh"
        if not self._runner.run_script(self._kindfile.scripts_dir(), script):
            return f"error running: {script}"

    def kill(self) -> None:
        self._kindfile.throw_if_no_kindfile_found()

        cluster_name = self._kindfile.cluster_name()
        if not self._runner.kind_is_running(cluster_name):
            print(f"cluster {cluster_name} is no running")
            return

        self._create_content()

        script = "shell-delete.sh"
        if not self._runner.run_script(self._kindfile.scripts_dir(), script):
            return f"error running: {script}"

    def _create_content(self) -> str:
        cfg_data = self._kindfile.data()
        self._tpl.render_template(cfg_data, "j2/shell-install.j2.sh", ".kind/scripts", "", 0o0755)
        self._tpl.render_template(cfg_data, "j2/shell-delete.j2.sh", ".kind/scripts", "", 0o0755)


from pathlib import Path
from typing import Dict
from jinja2 import Environment, FileSystemLoader
import os
import shutil

class Templates:
    def __init__(self, dest_dir: Path) -> None:
        self._tpl_path = self._detect_template_path()
        self._dest_dir = os.path.realpath(dest_dir)

    def _detect_template_path(self) -> str:
        # probably better ?
        # https://stackoverflow.com/questions/5897666/how-do-i-use-data-in-package-data-from-source-code
        tpl_dir_basename = "templates"
        file_path = os.path.realpath(__file__)
        file_dir = os.path.dirname(file_path)
        tpl_dir = os.path.join(file_dir, tpl_dir_basename)
        tpl_dir = os.path.abspath(tpl_dir)
        return tpl_dir

    def get_kindfile(self) -> str:
        kindfile = os.path.abspath(os.path.join(self._dest_dir, 'kindfile.yaml'))
        return kindfile

    def get_dest_dir(self) -> str:
        return self._dest_dir

    def copy_file(self, tpl_filename: str, dest_sub_dir: str="",
        mode: int=None) -> None:
        dest_file_path = self._dest_dir

        if dest_sub_dir:
            dest_file_path = os.path.join(self._dest_dir, dest_sub_dir)

        if not os.path.exists(dest_file_path):
            os.makedirs(dest_file_path)

        src_file = os.path.join(self._tpl_path, tpl_filename)

        dest_file = os.path.join(dest_file_path, os.path.basename(tpl_filename))
        dest_file = os.path.realpath(dest_file)

        shutil.copyfile(src_file, dest_file)

        if mode:
            try:
                Path(dest_file).chmod(mode)
            except Exception:
                # Windows friends
                pass

    def render_template(self, data: Dict[str,str], tpl_filename: str,
            dest_sub_dir: str, dest_filename: str="", mode: int=None
        ) -> None:
        dest_file_path = os.path.join(self._dest_dir, dest_sub_dir)

        if not dest_filename:
            dest_filename = os.path.basename(tpl_filename)
            dest_filename = dest_filename.replace(".j2", "")

        if not os.path.exists(dest_file_path):
            os.makedirs(dest_file_path)

        dest_file = os.path.realpath(os.path.join(dest_file_path, dest_filename))

        self._render_template(data, tpl_filename, dest_file)

        if mode:
            try:
                Path(dest_file).chmod(mode)
            except Exception:
                # Windows friends
                pass

    def delete_dot_kind_dir(self) -> None:
        kind_dir = os.path.abspath(os.path.join(self.get_dest_dir(), '.kind'))
        shutil.rmtree(kind_dir, ignore_errors=False)
        return None

    def delete_scripts_dir(self) -> None:
        dir = os.path.abspath(os.path.join(self.get_dest_dir(), '.kind/scripts'))
        if os.path.exists(dir):
            shutil.rmtree(dir, ignore_errors=False)
        return None

    def delete_config_dir(self) -> None:
        dir = os.path.abspath(os.path.join(self.get_dest_dir(), '.kind/config'))
        if os.path.exists(dir):
            shutil.rmtree(dir, ignore_errors=False)
        return None

    def _render_template(self, cfg_data: dict[str, str], tpl_filename: str, dest_file: str) -> None:
        # https://ttl255.com/jinja2-tutorial-part-1-introduction-and-variable-substitution/
        # https://stackoverflow.com/questions/69056354/access-jinja2-templates-from-a-folder-outside-of-package

        env = Environment(loader=FileSystemLoader(self._tpl_path))
        tmpl = env.get_template(tpl_filename)
        data = tmpl.render(cfg_data)

        Path(dest_file).write_text(data)


import subprocess
import os

from typing import List


class Runner:
    def __init__(self) -> None:
        self._kind_cmd = "kind"
        return None

    def kind_is_running(self, cluster_name: str) -> bool:
        # kind get cluster 2>/dev/null | grep ${cluster_name}

        cmd = [self._kind_cmd, "get", "clusters"]
        p = subprocess.Popen(cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.DEVNULL)
        out, _ = p.communicate()

        if p.returncode:
            return False

        if not out:
            return False

        out_str = out.decode('utf-8')
        res = cluster_name in out_str
        return res

    def kind(self, args: List[str]) -> bool:

        cmd = [self._kind_cmd]
        for arg in args:
            cmd.append(arg)

        p = subprocess.Popen(cmd)
        p.communicate()

        if p.returncode:
            return False

        return True

    def run_script(self, scripts_dir: str, script: str) -> bool:
        cmd = []
        cmd.append(os.path.abspath(os.path.join(scripts_dir, script)))

        p = subprocess.Popen(cmd, shell=True)
        p.communicate()

        if p.returncode:
            return False

        return True


import yaml
import os
from subprocess import check_output
from typing import Any, Dict

from kindtool import __version__, templates

class Kindfile:
    def __init__(self, tpl: templates.Templates, inject: Dict[str, str] = {}) -> None:
        self._tpl = tpl
        self._cfg = ClusterConfig(tpl)
        self._data = None
        self._inject = inject

    def throw_if_no_kindfile_found(self) -> None:
        kind_filename = self._tpl.get_kindfile()
        if not os.path.exists(kind_filename):
            raise FileNotFoundError(f"kindfile.yaml not found {kind_filename}")

    def data(self) -> Dict[str, str]:
        if not self._data:
            self._cfg.parse(self._inject)
            self._data = self._cfg.data()
        return self._data

    def has_config(self) -> bool:
        config_yml = self.config_yaml()
        res = os.path.exists(config_yml)
        return res

    def kubeconfig(self) -> str:
        config_dir = self.config_dir()

        if not config_dir:
            return ""

        res = os.path.abspath(os.path.join(config_dir, 'config'))
        return res

    def config_yaml(self) -> str:
        config_dir = self.config_dir()

        if not config_dir:
            return ""

        res = os.path.abspath(os.path.join(config_dir, 'config.yaml'))
        return res

    def scripts_dir(self) -> str:
        res = os.path.abspath(os.path.join(self._tpl.get_dest_dir(), '.kind/scripts'))
        return res

    def config_dir(self) -> str:
        key = 'config_dir'
        config_dir = self.get_raw(key)
        res = os.path.abspath(config_dir)
        return res

    def cluster_name(self) -> str:
        key = 'cluster_name'
        return self.get_raw(key)

    def has_local_kubeconfig(self) -> bool:
        key = 'local_kubeconfig'
        return self.get_raw(key)

    def has_internal_registry(self) -> bool:
        key = 'internal_registry'
        return self.get_raw(key)

    def has_ingress(self) -> bool:
        key = 'ingress'
        return self.get_raw(key)

    def has_metallb(self) -> bool:
        key = 'metallb'
        return self.get_raw(key)

    def get_raw(self, key: str) -> str:
        data = self.data()

        if key not in data:
            return ""

        res = data[key]
        return res

class ClusterConfig:
    def __init__(self, tpl: templates.Templates) -> None:
        self._tpl = tpl
        self._data = {}

    def parse(self, inject: Dict[str, Any] = {}) -> None:

        with open(self._tpl.get_kindfile(), "r") as stream:
            self._data = yaml.safe_load(stream)

        # inject data not in kindfile.yaml
        for key in inject:
            value = inject[key]
            self._data[key] = value

        # patch a few things
        self._update_api_server_address()
        self._update_config_dir()

        key = "mountpoints"
        if self.getboolean(key):
            self._update_mount_dir()

        key = "cluster_name"
        self._data[key] = self.get(key, "kind")

        key = "internal_registry"
        if self.getboolean(key):
            key = "internal_registry_docker_name"
            self._data[key] = self._data["cluster_name"]  + "-registry"
            key = "internal_registry_docker_port"
            self._data[key] = self.getint(key, 5001)

        key = "ingress"
        if self.getboolean(key):
            key = "ingress_http_port"
            self._data[key] = self.getint(key, 8000)
            key = "ingress_https_port"
            self._data[key] = self.getint(key, 8443)

        key = "worker_nodes"
        self._data[key] = self.getint(key, 0)

        # inject our version
        key = "kindtool_version"
        self._data[key] = __version__


    def data(self) -> dict[str, Any]:
        return self._data

    def get(self, key: str, default: str = "") -> str:
        if key in self._data:
            if self._data[key] != None and str(self._data[key]).lower() != "none":
                return self._data[key]
        self._data[key] = default
        return self._data[key]

    def getboolean(self, key: str, default: bool = False) -> bool:
        if key in self._data:
            if self._data[key] != None and str(self._data[key]).lower() != "none":
                return self._data[key]
        self._data[key] = default
        return self._data[key]

    def getint(self, key: str, default: int = 0) -> int:
        if key in self._data:
            if self._data[key] != None and str(self._data[key]).lower() != "none":
                return self._data[key]
        self._data[key] = default
        return self._data[key]

    def getfloat(self, key: str, default: float = 0) -> float:
        if key in self._data:
            if self._data[key] != None and str(self._data[key]).lower() != "none":
                return self._data[key]
        self._data[key] = default
        return self._data[key]

    def _update_config_dir(self) -> str:
        key = "config_dir"
        value = self.get(key, "")
        if not value:
            value = os.path.realpath(os.path.join(self._tpl.get_dest_dir(), ".kind/config"))
        #value = os.path.abspath(value)
        self._data[key] = value

    def _update_mount_dir(self) -> str:
        key = "mount_dir"
        value = self.get(key, "")
        if not value:
            value = os.path.realpath(os.path.join(self._tpl.get_dest_dir(), ".kind/data"))
            self._data[key] = value
            # this enforces our automatic generated dir to be 755 and our user
            os.makedirs(value, exist_ok=True)

    def _update_api_server_address(self) -> None:
        key = "api_server_address"
        value = self.get(key, "")
        if not value:
            value = self._get_ip()
            self._data[key] = value

    def _get_ip(self):
        ip = "127.0.0.1"
        try:
            # bash style of getting a "cool" ip address
            # will fail on macOS / Windows / and a lot of Linux versions - help is welcome!
            ips = check_output(['hostname', '--all-ip-addresses'])
            ips = ips.decode('ascii')
            ip_arr = ips.split()
            ip = ip_arr[0]
        except Exception:
            # bad luck...
            pass
        return ip



import argparse

from kindtool import __app_name__, __version__, cmdinit, cmdget, cmdup, cmddestroy, cmddashboard, cmdshell, templates

def add_default_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument( '--directory', '-d', type=str,
        metavar='DIR',
        default='',
        help="directory of kindfile.yaml (default is current working directory)", required=False)

def main_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog=__app_name__)
    parser.add_argument('--version', action='version',
                    version='%(prog)s {version}'.format(version=__version__))
    return parser

def create_parser_init(parent: argparse.ArgumentParser) -> None:
    name = 'init'
    help = 'create a new kindfile.yaml'

    parser = parent.add_parser(name, help=help)
    parser.add_argument( '--directory', '-d', type=str,
        metavar='DIR',
        default='',
        help="destination directory (default is current working directory)", required=False)

def create_parser_up(parent: argparse.ArgumentParser) -> None:
    name = 'up'
    help = 'start a cluster'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_destroy(parent: argparse.ArgumentParser) -> None:
    name = 'destroy'
    help = 'stops a cluster'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)
    parser.add_argument('--force','-f', action='store_true',
        help="force the deletion")

def create_parser_status(parent: argparse.ArgumentParser) -> None:
    name = 'status'
    help = 'prints status information of the cluster'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_dashboard(parent: argparse.ArgumentParser) -> None:
    name = 'dashboard'
    help = 'installs and start the k8s dashboard in the cluster'
    version = '2.6.1'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

    parser.add_argument( '--version', '-v', type=str,
        metavar='VER',
        default=version,
        help=f"version of the dashboard (default {version})", required=False)

def create_parser_shell(parent: argparse.ArgumentParser) -> None:
    name = 'shell'
    help = 'installs and start the k8s dashboard in the cluster'
    pod = 'shell-pod'
    image = 'ubuntu:22.04'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

    parser.add_argument( '--pod', '-p', type=str,
        metavar='POD',
        default=pod,
        help=f"name of the shell pod (default {pod})", required=False)
    parser.add_argument( '--image', '-i', type=str,
        metavar='IMAGE',
        default=image,
        help=f"docker image of the shell pod (default {image})", required=False)
    parser.add_argument( '--namespace', '-n', type=str,
        metavar='NS',
        default="default",
        help=f"name of the namespace", required=False)
    parser.add_argument( '--cmd', '-c', type=str,
        metavar='CMD',
        default="",
        help=f"command to run as shell (e.g. /bin/bash - default guesstimate shell)", required=False)
    parser.add_argument('--kill','-k', action='store_true',
        help="kill shell pod")


def create_parser_get_name(parent: argparse.ArgumentParser) -> None:
    name = 'name'
    help = 'name of the cluster'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_kubeconfig(parent: argparse.ArgumentParser) -> None:
    name = 'kubeconfig'
    help = 'returns the directory containing the kubeconfig of the cluster'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_ingress(parent: argparse.ArgumentParser) -> None:
    name = 'ingress'
    help = 'returns True or False if ingress feature is enabled'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_ingress_http_port(parent: argparse.ArgumentParser) -> None:
    name = 'ingress_http_port'
    help = 'returns the ingress http port'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_ingress_https_port(parent: argparse.ArgumentParser) -> None:
    name = 'ingress_https_port'
    help = 'returns the ingress https port'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_metallb(parent: argparse.ArgumentParser) -> None:
    name = 'metallb'
    help = 'returns True or False if metallb feature is enabled'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_mountpoints(parent: argparse.ArgumentParser) -> None:
    name = 'mountpoints'
    help = 'returns True or False if the cluster has local persistant storage'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_api_server_address(parent: argparse.ArgumentParser) -> None:
    name = 'api_server_address'
    help = 'returns the ip address of the k8s API server'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def create_parser_get_internal_registry_prefix(parent: argparse.ArgumentParser) -> None:
    name = 'internal_registry_prefix'
    help = 'returns the prefix internal docker registry if internal_registry is enabled'

    parser = parent.add_parser(name, help=help)
    add_default_arguments(parser)

def main() -> None:
    parser = main_parser()
    subparser = parser.add_subparsers(dest='command')
    create_parser_init(subparser)
    create_parser_up(subparser)
    create_parser_destroy(subparser)
    create_parser_status(subparser)
    create_parser_dashboard(subparser)
    create_parser_shell(subparser)

    # 'get' has subcommands
    parser_get = subparser.add_parser('get', help='get useful status information of the cluster')
    subparser = parser_get.add_subparsers(dest='get')

    create_parser_get_name(subparser)
    create_parser_get_kubeconfig(subparser)
    create_parser_get_ingress(subparser)
    create_parser_get_ingress_http_port(subparser)
    create_parser_get_ingress_https_port(subparser)
    create_parser_get_metallb(subparser)
    create_parser_get_mountpoints(subparser)
    create_parser_get_api_server_address(subparser)
    create_parser_get_internal_registry_prefix(subparser)

    args = parser.parse_args()

    if args.command == 'init':
        tpl = templates.Templates(dest_dir=args.directory)
        cmd = cmdinit.CmdInit(tpl)
        cmd.create_content()
    elif args.command == 'up':
        tpl = templates.Templates(dest_dir=args.directory)
        cmd = cmdup.CmdUp(tpl)
        cmd.run()
    elif args.command == 'destroy':
        tpl = templates.Templates(dest_dir=args.directory)
        cmd = cmddestroy.CmdDestroy(tpl)
        cmd.run(args.force)
    elif args.command == 'status':
        raise NotImplementedError(f"command '{args.command}' is not implemented")
    elif args.command == 'dashboard':
        tpl = templates.Templates(dest_dir=args.directory)
        cmd = cmddashboard.CmdDashboard(tpl, args.version)
        cmd.run()
    elif args.command == 'shell':
        tpl = templates.Templates(dest_dir=args.directory)
        cmd = cmdshell.CmdShell(tpl, args.pod, args.namespace, args.image, args.cmd)
        if args.kill:
            cmd.kill()
        else:
            cmd.create_or_attach()
    elif args.command == 'get':
        arr = [
            'name','kubeconfig',
            'ingress', 'ingress_http_port', 'ingress_https_port',
            'metallb',
            'mountpoints',
            'api_server_address',
            'internal_registry_prefix'
        ]
        if args.get in arr:
            tpl = templates.Templates(dest_dir=args.directory)
            cmd = cmdget.CmdGet(tpl)
            cmd.get(args.get)
        else:
            parser_get.print_usage()
    else:
        parser.print_usage()


"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib
from glob import glob

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name="kindtool",  # Required
    version="0.0.4",  # Required
    description="Generator for kind k8s clusters",  # Optional
    long_description=long_description,  # Optional
    long_description_content_type="text/markdown",  # Optional (see note above)
    url="https://github.com/egandro/kindtool",  # Optional
    author="Harald Fielker",  # Optional
    author_email="harald.fielker@gmail.com",  # Optional
    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        # Pick your license as you wish
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="kind, k8s, generator, development",  # Optional
    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    package_dir={"": "src"},  # Optional
    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(where="src"),  # Required
    python_requires=">=3.7, <4",
    install_requires=[
        "argparse",
        "Jinja2",
        "pyyaml"
    ],
    extras_require={  # Optional
        "dev": ["check-manifest"],
        "test": ["coverage"],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #package_data={  # Optional
    #    "sample": ["package_data.dat"],
    #},
    package_data={
        "kindtool": [ "templates/**", "templates/**/*"]
    },

    # # https://stackoverflow.com/questions/27829754/include-entire-directory-in-python-setup-py-data-files
    # data_files=[("kindtool_templates",
    #     glob('kindtool_templates/**/*', recursive=True), # includes sub-folders - recursive
    # )],  # Optional
    entry_points={  # Optional
        "console_scripts": [
            "kindtool=kindtool.__main__:main"
        ],
    },
    project_urls={  # Optional
        "Bug Reports": "https://github.com/egandro/kindtool/issues",
        "Source": "https://github.com/egandro/kindtool/",
    },
)



from typing import Any, Dict

from kindtool import __version__, runner, templates, kindfile

class CmdDestroy:
    def __init__(self, tpl: templates.Templates) -> None:
        self._tpl = tpl
        self._kindfile = kindfile.Kindfile(tpl)
        self._runner = runner.Runner()

    def run(self, force: bool) -> None:
        self._kindfile.throw_if_no_kindfile_found()

        if not self._kindfile.has_config():
            # we never started
            return

        args = [
            "delete",
            "cluster",
            "--name", self._kindfile.cluster_name()
        ]

        if not self._runner.kind(args):
            return "can't delete the cluster"

        if self._kindfile.has_internal_registry():
            script = "internal-registry-delete.sh"
            if not self._runner.run_script(self._kindfile.scripts_dir(), script):
                return f"error running: {script}"

        self._tpl.delete_scripts_dir()
        self._tpl.delete_config_dir()

        if force:
            self._tpl.delete_dot_kind_dir()




from kindtool import templates, kindfile
import os

class CmdGet:
    def __init__(self, tpl: templates.Templates) -> None:
        self._tpl = tpl
        self._kindfile = kindfile.Kindfile(tpl)

    def get(self, key: str) -> None:
        self._kindfile.throw_if_no_kindfile_found()

        value=''

        if key == 'name':
            value = self._kindfile.cluster_name()
        elif key == 'kubeconfig':
            if self._kindfile.has_local_kubeconfig():
                value = os.path.abspath(os.path.join(self._kindfile.config_dir(), 'config'))
        elif key == 'ingress':
            value = f"{self._kindfile.has_ingress()}"
        elif key == 'ingress_http_port':
            if self._kindfile.has_ingress():
                value = self._kindfile.get_raw('ingress_http_port')
        elif key == 'ingress_https_port':
            if self._kindfile.has_ingress():
                value = self._kindfile.get_raw('ingress_https_port')
        elif key == 'metallb':
            value = f"{self._kindfile.has_metallb()}"
        elif key == 'mountpoints':
            value = self._kindfile.get_raw('mountpoints')
        elif key == 'api_server_address':
            value = self._kindfile.get_raw('api_server_address')
        elif key == 'internal_registry_prefix':
            if self._kindfile.has_internal_registry():
                value = f"localhost:{self._kindfile.get_raw('internal_registry_docker_port')}"
        else:
            raise NotImplementedError(f"key '{key}' not implemented")

        print(value)